# GnuPGverify.pm
#
# A module for checking GnuPG signatures.
#
# author:  Max-Gerd Retzlaff <m.retzlaff@gmx.net>
#
#          parts of the GnuPG-Code in check_signature()
#             by Reyk Floeter <reyk@synack.de>
#
# date:    $Date: 2002/02/06 11:51:52 $
# version: $Revision: 1.3 $
#
# This application is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version. 
#
# This application is distributed in the hope that it will be useful, 
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Library General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this library; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
# 02111 USA.  

package GnuPGverify;

use HelpfulFunctions; 

use GnuPG::Interface;
if ( defined(%ENV) && exists($ENV{HOME}) ) {
    $home_dir = "$ENV{HOME}";
} else {
    $home_dir = "/tmp";
}
$gnupg_dir = "$home_dir/.gnupg";
%valid_IDs = ();

sub set_gnupg_dir {
    my ($pkg, $dir) = @_;
    $gnupg_dir = $dir;
}

sub set_home_dir {
    my ($pkg, $dir) = @_;
    $home_dir = $dir;
}

sub load_valid_IDs {
    my ($pkg, $filename) = @_;

    do $filename;
}

sub get_valid_IDs {
    return %valid_IDs;
}

sub check_signature {
    my ($pkg, $mail_type, $content_part, $gpg_signature) = @_;
    
    use IO::Handle;
    use GnuPG::Interface;
    use File::Temp;

    my $gnupg = GnuPG::Interface->new();
    $gnupg->options->hash_init(armor => 1, 
			       textmode => 1,
			       homedir => $gnupg_dir);
    $gnupg->options->meta_interactive(0); # suppress some babbling...

    my $input = IO::Handle->new();
    my $error = IO::Handle->new();
    my $output = IO::Handle->new();
    my $handles = GnuPG::Handles->new(stdin => $input, 
				      stderr => $error,
				      stdout => $output);

    my ($sigfh, $sigfile) = 
      File::Temp::tempfile('/tmp/mwiki-sigXXXXX', UNLINK => 1);
    print $sigfh $gpg_signature;

    $gnupg->verify(handles => $handles,  
		   command_args => [ "$sigfile", "-" ] );

    print $input $content_part;
    close $sigfh;
    close $input;
    close $output;

    my @out = <$error>;
    close $error;

    # print @out,"\n";

    my $sign_ID;
    if ($out[1] =~ /^gpg: Good signature/i) {
	$sign_ID = $out[0];
	$sign_ID =~ s/^.*?\sID\s([^\s]*)$/0x$1/gi;
	chomp $sign_ID;
    } else {
	mdie(0, "ERROR: gpg: BAD signature\n");
	return 1;
    }

    foreach $valid_ID (@{$valid_IDs{$mail_type}}) {
	if ( $valid_ID eq $sign_ID ) {
	    return 0; # access granted
	}
    }
    
    mdie(0, "ERROR: access denied for $sign_ID (for messages of type $mail_type)\n");
    return 2;
}

return 1; # Last executing statement in file must be
          # non-zero, to indicate succesful loading
